# -*- encoding: utf-8 -*-
# Copyright (c) 2019 ZTE Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

keystone_client = cfg.OptGroup(name='keystone_client',
                               title='Configuration Options for Keystone')

KEYSTONE_CLIENT_OPTS = [
    cfg.StrOpt('interface',
               default='admin',
               choices=['internal', 'public', 'admin'],
               help='Type of endpoint to use in keystoneclient.'),
    cfg.StrOpt('region_name',
               help='Region in Identity service catalog to use for '
                    'communication with the OpenStack service.')]


def register_opts(conf):
    conf.register_group(keystone_client)
    conf.register_opts(KEYSTONE_CLIENT_OPTS, group=keystone_client)


def list_opts():
    return [(keystone_client, KEYSTONE_CLIENT_OPTS)]
